#include <iostream>
#include <vector>
#include <deque>
#include <unordered_map>
#include <set>
#include <map>
#include <cassert>
#include <limits>
#include <algorithm>

struct Edge
{
    int v0, v1;
    int64_t scenic;
    std::vector<int> children;
    bool boundary;    
};

int otherVert(int src, const Edge& e)
{
    if (e.v0 == src)
        return e.v1;
    else
        return e.v0;
}

int main()
{
    int V, E;
    std::cin >> V >> E;

    std::vector<std::set<int> > vertexEdges(V);
    std::vector<Edge> edges(E);
    std::vector<std::set<int> > vertexNeighbors(V);

    for (int i = 0; i < E; i++)
    {
        int a, b;
        int64_t s;
        std::cin >> a >> b >> s;
        if (a > b) std::swap(a, b);

        edges[i].v0 = a - 1;
        edges[i].v1 = b - 1;
        edges[i].scenic = s;
        edges[i].boundary = false;
        vertexEdges[a - 1].insert(i);
        vertexEdges[b - 1].insert(i);
        vertexNeighbors[a - 1].insert(b - 1);
        vertexNeighbors[b - 1].insert(a - 1);
    }

    std::deque<int> verticesToCollapse;

    for (int i = 0; i < V; i++)
    {
        if (vertexNeighbors[i].size() <= 2)
            verticesToCollapse.push_back(i);
    }

    int64_t best = -std::numeric_limits<int64_t>::max();

    while (!verticesToCollapse.empty())
    {
        int next = verticesToCollapse.front();
        verticesToCollapse.pop_front();
        if (vertexNeighbors[next].size() == 1)
        {
            std::vector<int64_t> scenics;
            for (auto it : vertexEdges[next])
                scenics.push_back(edges[it].scenic);
            std::sort(scenics.begin(), scenics.end());
            int64_t answer = scenics[scenics.size() - 2] + scenics[scenics.size() - 1];
            best = std::max(best, answer);
            std::cout << best << std::endl;
            return 0;
        }
        else if (vertexNeighbors[next].size() != 2)
            continue;
        
        int nb[2];
        nb[0] = *vertexNeighbors[next].begin();
        nb[1] = *(++vertexNeighbors[next].begin());

        std::vector<int> nbedges[2];
        for (auto it : vertexEdges[next])
        {
            if (otherVert(next, edges[it]) == nb[0])
                nbedges[0].push_back(it);
            else if (otherVert(next, edges[it]) == nb[1])
                nbedges[1].push_back(it);
        }

        bool newEdgeBoundary = false;
        int keptedge[2];

        int64_t newscenic = 0;

        for (int i = 0; i < 2; i++)
        {
            std::vector<int> interiorEdges;
            std::vector<int> boundaryEdges;
            for (auto it : nbedges[i])
            {
                if (edges[it].boundary)
                    boundaryEdges.push_back(it);
                else
                    interiorEdges.push_back(it);
            }
            if (boundaryEdges.size() == 1)
            {
                newEdgeBoundary = true;
                interiorEdges.push_back(boundaryEdges[0]);
            }

            keptedge[i] = interiorEdges[0];
            if (interiorEdges.size() > 1)
                newEdgeBoundary = true;
            
            std::vector<int64_t> scenics;

            int64_t bests = -std::numeric_limits<int64_t>::max();

            for (auto it : interiorEdges)
            {
                vertexEdges[next].erase(it);
                vertexEdges[nb[i]].erase(it);
                scenics.push_back(edges[it].scenic);
                bests = std::max(bests, edges[it].scenic);
            }

            if (scenics.size() > 1)
            {
                std::sort(scenics.begin(), scenics.end());
                int64_t answer = scenics[scenics.size() - 2] + scenics[scenics.size() - 1];
                best = std::max(best, answer);
            }

            newscenic += bests;
        }

        Edge newe;
        newe.v0 = nb[0];
        newe.v1 = nb[1];
        newe.boundary = newEdgeBoundary;
        newe.children = { keptedge[0], keptedge[1] };
        newe.scenic = newscenic;

        int neweid = edges.size();
        edges.push_back(newe);

        vertexEdges[nb[0]].insert(neweid);
        vertexEdges[nb[1]].insert(neweid);
        vertexNeighbors[nb[0]].erase(next);
        vertexNeighbors[nb[1]].erase(next);
        vertexNeighbors[nb[0]].insert(nb[1]);
        vertexNeighbors[nb[1]].insert(nb[0]);
        vertexNeighbors[next].clear();
        if (vertexNeighbors[nb[0]].size() <= 2)
            verticesToCollapse.push_back(nb[0]);
        if (vertexNeighbors[nb[1]].size() <= 2)
            verticesToCollapse.push_back(nb[1]);
    }


}
