import java.io.BufferedReader;
import java.util.*;

public class heaviestcycle_dg {

    static boolean debugMode = false;

    public static void main(String[] args) {
        Scanner in = new Scanner(System.in);

        while(true) {
            int V = in.nextInt();
            if (V == -1) break;
            if (V == -2) {
                debugMode = true;
                V = in.nextInt();
            }
            else {
                debugMode = false;
            }
            int E = in.nextInt();

            TreeSet<Edge>[] adj = new TreeSet[V];
            for(int v=0;v<V;v++) {
                adj[v] = new TreeSet<Edge>(forDualChecker);
            }

            for(int e = 0; e < E; e++) {
                int a = in.nextInt()-1;
                int b = in.nextInt()-1;
                long w = in.nextInt();
                Edge edge = new Edge(a,b,w);
                adj[a].add(edge);
                adj[b].add(edge);
            }

            long ans = go(V, adj);
            System.out.println(ans);

            // comment this next line when testing cases manually.
            break;
        }
    }

    static long go(int V, TreeSet<Edge>[] tradj) {
        // add all nodes to a "node check" set
        HashSet<Integer> nodeCheck = new HashSet<>();
        for(int v=0;v<V;v++) {
            nodeCheck.add(v);
        }

        // add all edges to an "edge check" set
        TreeSet<Edge> edgeCheck = new TreeSet<>(forDualChecker);
        for(int v=0;v<V;v++) {
            for (Edge e : tradj[v]) {
                edgeCheck.add(e);
            }
        }

        long biggestMerge = Long.MIN_VALUE;

        TreeSet<Integer> nodesOfBiggestCycle = null;

        // while the edge check set is not empty
        while (!edgeCheck.isEmpty()) {
            // for each edge in the edge check set
            while (!edgeCheck.isEmpty()) {
                TreeSet<Edge> moreEdgeCheck = new TreeSet<Edge>(forDualChecker);
                for (Edge e : edgeCheck) {
                    if (e.removed) continue;

                    // look for a duplicate edge.
                    int u = e.a, v = e.b;
                    Edge prev = tradj[u].lower(e);
                    Edge next = tradj[u].higher(e);
                    Edge with = null;
                    if (prev != null && prev.b == e.b) {
                        with = prev;
                    }
                    if (with == null && next != null && next.b == e.b) {
                        with = next;
                    }

                    // if we found a duplicate edge
                    if (with != null) {
                        // remove BOTH edges
                        tradj[u].remove(e);
                        tradj[v].remove(e);
                        tradj[u].remove(with);
                        tradj[v].remove(with);
                        e.removed = true;
                        with.removed = true;

                        if (debugMode && biggestMerge < e.w + with.w) {
                            nodesOfBiggestCycle = new TreeSet<>();
                            nodesOfBiggestCycle.addAll(e.treeSet);
                            nodesOfBiggestCycle.addAll(with.treeSet);
                        }

                        biggestMerge = Math.max(biggestMerge, e.w + with.w);
                        Edge merged = new Edge(u,v, Math.max(e.w, with.w));

                        // DEBUG PURPOSES ONLY.
                        if (debugMode) merged.treeSet = (e.w > with.w)? e.treeSet: with.treeSet;

                        // add the merged edge
                        tradj[u].add(merged);
                        tradj[v].add(merged);
                        moreEdgeCheck.add(merged);

                        // add its nodes to the node check set
                        nodeCheck.add(u);
                        nodeCheck.add(v);
                    }
                }
                edgeCheck = moreEdgeCheck;
            }
            edgeCheck = new TreeSet<Edge>(forDualChecker);

            // for each node in the node check set
            for (Integer io : nodeCheck) {
                // if it has two neighbors
                int i = io.intValue();
                if (tradj[i].size() == 2) {
                    // collapse it
                    Edge eu = tradj[i].first();
                    Edge ev = tradj[i].last();

                    int u = eu.other(i), v = ev.other(i);
                    if (u == v) continue;

                    tradj[i].remove(eu);
                    tradj[i].remove(ev);
                    tradj[u].remove(eu);
                    tradj[v].remove(ev);

                    // remove its edges from the edge check set (if applicable)
                    edgeCheck.remove(ev); edgeCheck.remove(eu);

                    // make a new edge between its former neighbors
                    Edge euv = new Edge(u, v, eu.w + ev.w);

                    // ONLY FOR DEBUG PURPOSES
                    if (debugMode) euv.mergeNodeSets(eu,ev);

                    tradj[u].add(euv);
                    tradj[v].add(euv);

                    // add the new edge to the edge check set
                    edgeCheck.add(euv);
                }
            }
            nodeCheck = new HashSet<Integer>();
        }

        // you need the biggest edge and second biggest edge to form one cycle
        for(int v = 0; v < V; v++) {
            long maxInHere = Long.MIN_VALUE / 3;
            Edge eBig = null;
            if (!tradj[v].isEmpty()) {

                for(Edge e : tradj[v]) {
                    long oneProp = e.w + maxInHere;

                    if (debugMode && biggestMerge < oneProp) {
                        nodesOfBiggestCycle = new TreeSet<>();
                        nodesOfBiggestCycle.addAll(e.treeSet);
                        nodesOfBiggestCycle.addAll(eBig.treeSet);
                    }

                    biggestMerge = Math.max(biggestMerge, oneProp);

                    if (debugMode && maxInHere < e.w) {
                        eBig = e;
                    }
                    maxInHere = Math.max(maxInHere, e.w);
                }
            }
        }

        if (debugMode && nodesOfBiggestCycle != null) {
            printSet(nodesOfBiggestCycle);
        }
        return biggestMerge;
    }

    static void printSet(TreeSet<Integer> nodesOfBiggestCycle) {
        StringBuilder sb = new StringBuilder();
        boolean first = true;
        for(Integer i : nodesOfBiggestCycle) {
            if (first) {
                first = false;
            } else {
                sb.append(' ');
            }
            sb.append(i+1);
        }
        System.out.println(sb);
    }

    static class Edge {
        // normal edge stuff
        static int numEdges = 0;
        int a, b, id;
        long w;
        TreeSet<Integer> treeSet;

        boolean removed = false;

        Edge(int aa, int bb, long ww) {
            if (aa > bb) {
                aa ^= bb; bb ^= aa; aa ^= bb;
            }
            if (debugMode) {
                treeSet = new TreeSet<>();
                treeSet.add(aa);
                treeSet.add(bb);
            }
            a=aa;b=bb; w=ww;
            id = numEdges;
            numEdges++;
        }
        int other(int o) {
            return o^a^b;
        }
        int sum() {
            return a+b;
        }

        void mergeNodeSets(Edge e1, Edge e2) {
            if (e1.treeSet.size() > e2.treeSet.size()) {
                Edge e = e1; e1 = e2; e2 = e;
            }
            treeSet = e2.treeSet;
            treeSet.addAll(e1.treeSet);
        }
    }

    static Comparator<Edge> forDualChecker = (a, b) -> {
        int sumA = a.sum(), sumB = b.sum();
        if (sumA != sumB) {
            return Integer.compare(sumA, sumB);
        }
        return Integer.compare(a.id, b.id);
    };
}